"use strict";
/* eslint
    complexity:"off",
    no-console:"off",
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.createStreamPreTokenizer = void 0;
const createPreTokenizer_1 = require("./createPreTokenizer");
const Whitespace = {
    tab: 0x09,
    lineFeed: 0x0A,
    carriageReturn: 0x0D,
    space: 0x20, //
};
/**
 *
 * @param tokenStreamConsumer
 * @param onError
 * @param opt
 */
function createStreamPreTokenizer(tokenStreamConsumer, onError, opt) {
    const location = {
        position: -1,
        column: 0,
        line: 1,
    };
    const spacesPerTab = opt === undefined
        ? 4
        : opt.spaces_per_tab === undefined
            ? 4
            : opt.spaces_per_tab;
    const locationState = {
        getCurrentLocation: () => {
            return {
                position: location.position + 1,
                line: location.line,
                column: location.column + 1,
            };
        },
        getNextLocation: () => {
            return {
                position: location.position + 2,
                line: location.line,
                column: location.column + 2,
            };
        },
        increase: (character) => {
            location.position++;
            //set the position
            switch (character) {
                case Whitespace.lineFeed:
                    location.line++;
                    location.column = 0;
                    break;
                case Whitespace.carriageReturn:
                    break;
                case Whitespace.tab:
                    location.column += spacesPerTab;
                    break;
                default:
                    location.column++;
            }
        },
    };
    const tokenizerState = (0, createPreTokenizer_1.createPreTokenizer)(locationState, onError);
    return {
        onData: (chunk) => {
            let currentIndex = 0;
            const str = chunk;
            const currentChunk = {
                lookahead: () => {
                    const char = str.charCodeAt(currentIndex);
                    return isNaN(char) ? null : char;
                },
                increaseIndex: () => {
                    currentIndex += 1;
                },
                getCurrentIndex: () => {
                    return currentIndex;
                },
                getString: () => {
                    return str;
                },
            };
            while (true) {
                const la = currentChunk.lookahead();
                if (la === null) {
                    return;
                }
                function createLoopState(chunk2) {
                    let startIndex = null;
                    function ensureFlushed(callback) {
                        if (startIndex !== null) {
                            return {
                                startSnippet: false,
                                consumeCharacter: false,
                                preToken: {
                                    type: ["snippet", {
                                            chunk: chunk2.getString(),
                                            begin: startIndex,
                                            end: chunk2.getCurrentIndex(),
                                        }],
                                },
                            };
                        }
                        return callback();
                    }
                    return {
                        /**
                         * if not flushed, the callback is not called.
                         * the current character position should not change so that the next round
                         * the same call will be made, but now it is flushed, so the callback will be called
                         */
                        ensureFlushed: (callback) => {
                            return ensureFlushed(callback);
                        },
                        whileLoop: (callback) => {
                            function whileLoop(callback2) {
                                while (true) {
                                    const returnValue = callback2();
                                    if (returnValue !== undefined) {
                                        return returnValue;
                                    }
                                }
                            }
                            return whileLoop(() => {
                                const nextChar = chunk2.lookahead();
                                if (nextChar === null) {
                                    return ensureFlushed(() => {
                                        return {
                                            startSnippet: false,
                                            consumeCharacter: false,
                                            preToken: null,
                                        };
                                    }).preToken;
                                }
                                const result = callback(nextChar);
                                if (result.startSnippet) {
                                    if (startIndex === null) {
                                        startIndex = chunk2.getCurrentIndex();
                                    }
                                }
                                if (result.consumeCharacter) {
                                    const cc = chunk2.lookahead();
                                    if (cc === null) {
                                        throw new Error("Unexpected consume");
                                    }
                                    locationState.increase(cc);
                                    chunk2.increaseIndex();
                                }
                                if (result.preToken !== null) {
                                    return result.preToken;
                                }
                                return undefined;
                            });
                        },
                    };
                }
                const loopState = createLoopState(currentChunk);
                const tokenData = tokenizerState.createNextToken(loopState);
                if (tokenData !== null) {
                    tokenStreamConsumer.onData(tokenData);
                }
            }
        },
        onEnd: () => {
            const tokenData = tokenizerState.handleDanglingToken();
            if (tokenData !== null) {
                tokenStreamConsumer.onData(tokenData);
            }
            tokenStreamConsumer.onEnd(locationState.getCurrentLocation());
        },
    };
}
exports.createStreamPreTokenizer = createStreamPreTokenizer;
